; Simple array Index routine
; Number of total indexes dimensions - 1 at beginning of memory
; HL = Start of array memory (First two bytes contains N-1 dimensions)
; Dimension values on the stack, (top of the stack, highest dimension)
; E.g. A(2, 4) -> PUSH <4>; PUSH <2>

; For any array of N dimension A(aN-1, ..., a1, a0)
; and dimensions D[bN-1, ..., b1, b0], the offset is calculated as
; O = [a0 + b0 * (a1 + b1 * (a2 + ... bN-2(aN-1)))]
; What I will do here is to calculate the following sequence:
; ((aN-1 * bN-2) + aN-2) * bN-3 + ...


#include once <mul16.asm>

__ARRAY:
	PROC

	LOCAL LOOP
	LOCAL ARRAY_END
	LOCAL RET_ADDRESS ; Stores return address

	pop de		; Return address
	ld (RET_ADDRESS), de ; Stores it for later

	push hl		; Indexes pointer goes to H'L'

	exx
	pop hl		; Will use H'L' as the pointer
	ld c, (hl)	; Loads Number of dimensions from (hl)
	inc hl
	ld b, (hl)
	inc hl		; Ready
	exx
		
	ld hl, 0	; BC = Offset "accumulator"
LOOP:
	pop bc		; Get next index (Ai) from the stack
	add hl, bc	; Adds current index

	exx			; Checks if B'C' = 0
	ld a, b		; Which means we must exit (last element is not multiplied by anything)
	or c
	exx
	jp z, ARRAY_END		; if B'Ci == 0 we are done

	exx					; H'L' points to next dimension.
	ld e, (hl)			; Loads next dimension into D'E'
	inc hl
	ld d, (hl)
	inc hl
	push de
	dec bc				; Decrements loop counter
	exx
	pop de				; DE = Max bound Number (i-th dimension)

	call __MUL16_FAST	; HL *= DE
	jp LOOP
	
ARRAY_END:
	exx
	;	push hl		; Current elements start
	ld e, (hl)
	inc hl
	ld d, c			; C = 0 => DE = E = Element size
	push hl
	push de
	exx
	pop de
	call __MUL16_FAST	; Multiplies by element size
	pop de
	add hl, de  ; Adds element start

	ld de, (RET_ADDRESS)
	push de
	ret			; HL = (Start of Elements + Offset)

RET_ADDRESS	EQU	23563	; DEFADD variable. 

	ENDP
	
